import React, { useEffect, useState } from 'react'
import { withClient, Client } from 'cozy-client'
import { translate } from 'cozy-ui/react/I18n'

import { isKonnectorRunning } from 'cozy-harvest-lib/dist/helpers/triggers'
import {
  KonnectorJob,
  ERROR_EVENT,
  LOGIN_SUCCESS_EVENT,
  SUCCESS_EVENT,
} from 'cozy-harvest-lib/dist/models/KonnectorJob'
import { Trigger } from 'doctypes'

import Lottie from 'react-lottie'
import * as loadingData from 'assets/anims/bounceloading.json'
import StyledButton from 'components/CommonKit/Button/StyledButton'
import Modal from 'components/CommonKit/Modal/Modal'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import errorIcon from 'assets/icons/visu/data-nok.svg'
import successIcon from 'assets/icons/visu/data-ok.svg'

const loadingOptions = {
  loop: true,
  autoplay: true,
  animationData: loadingData,
  rendererSettings: {
    preserveAspectRatio: 'xMidYMid slice',
  },
}

interface KonnectorLaunchProps {
  trigger: Trigger
  type: string
  handleKonnectorLaunch: Function
  client: Client
  t: Function
}

const KonnectorLaunch: React.FC<KonnectorLaunchProps> = ({
  trigger,
  type,
  handleKonnectorLaunch,
  client,
  t,
}: KonnectorLaunchProps) => {
  const [state, setState] = useState<string | null>(null)
  const [openModal, setOpenModal] = useState(false)

  const callbackResponse = (_state: string) => {
    setState(_state)
  }

  const handleCloseClick = () => {
    handleKonnectorLaunch()
    setOpenModal(false)
  }

  useEffect(() => {
    let subscribed = true
    async function getData() {
      if (subscribed && !isKonnectorRunning(trigger)) {
        const konnectorJob = new KonnectorJob(client, trigger)
        await konnectorJob.launch()
        konnectorJob.jobWatcher.on(ERROR_EVENT, () => {
          callbackResponse(ERROR_EVENT)
        })
        konnectorJob.jobWatcher.on(LOGIN_SUCCESS_EVENT, () => {
          callbackResponse(LOGIN_SUCCESS_EVENT)
        })
        konnectorJob.jobWatcher.on(SUCCESS_EVENT, () => {
          callbackResponse(SUCCESS_EVENT)
        })
        setOpenModal(true)
      }
    }
    getData()
    return () => {
      subscribed = false
    }
  }, [])

  return (
    <Modal open={openModal} handleCloseClick={handleCloseClick}>
      <div className="klaunch-content">
        {!state ? (
          <>
            <Lottie options={loadingOptions} height={50} width={50} speed={2} />
            <div className="klaunch-content-text klaunch-content-text-center text-16-normal">
              <div className="kc-wait">{t('KONNECTORCONFIG.PLZ_WAIT')}</div>
            </div>
            <div className="klaunch-content-text text-16-normal">
              <div>{t('KONNECTORCONFIG.LOADING_DATA')}</div>
            </div>
          </>
        ) : (
          <>
            <div className="klaunch-info">
              {state === ERROR_EVENT ? (
                <div className="konnector-config">
                  <StyledIcon icon={errorIcon} size={48} />
                  <div className="kce-picto-txt text-20-bold">
                    {t('KONNECTORCONFIG.ERROR_TXT')}
                  </div>
                  <div>{t('KONNECTORCONFIG.ERROR_DATA_' + type)}</div>
                  <div>{t('KONNECTORCONFIG.ERROR_DATA_2')}</div>
                </div>
              ) : (
                <div className="konnector-config">
                  <StyledIcon icon={successIcon} size={48} />
                  <div className="kcs-picto-txt text-20-bold">
                    {t('KONNECTORCONFIG.SUCCESS_TXT')}
                  </div>
                  {t('KONNECTORCONFIG.SUCCESS_DATA_' + type)}
                </div>
              )}
              <StyledButton
                type="button"
                color="primary"
                onClick={handleCloseClick}
              >
                <div>{t('KONNECTORCONFIG.OK')}</div>
              </StyledButton>
            </div>
          </>
        )}
      </div>
    </Modal>
  )
}

export default translate()(withClient(KonnectorLaunch))
